@echo off

rem FILE: Ascii.bat  ( App - Ascii )
rem COPYRIGHT: (c), Symmetric Research, 2013-2019
rem
rem This is a demo program.  It shows how to set up a pipeline to bring data
rem all the way from the USBxCH board to a text screen display.
rem
rem "The program" is actually this batch file.  Three pipeline utilities are
rem used: ( Blast , Pak2Bin , AsciiDisplay ).  Data moves from one utility to
rem the next via intermediate named pipes.
rem
rem Once you are comfortable with how this batch file works, more complicated
rem batch files can be found in the "App - DVM" and "App - Scope" directories.
rem
rem See the "Examples" directory for pipelines aimed at specific applications.
rem 
rem Documentation for all of the pipeline utilities can be found in the "Docs"
rem directory.  Executables are in the "Exe" directory.
rem
rem The comments below have further info about the batch file constructs used.
rem
rem
rem
rem *** TWEAKING AsciiDisplay CONSOLE SIZE/WIDTH:
rem
rem For the AsciiDisplay.exe console you may want to set the font and line
rem width to avoid line wrapping.  To do so:
rem
rem  * right click on the AsciiDisplay.exe title bar
rem  * select "Properties"
rem  * on the "Layout tab", set the screen buffer width to 220 ( characters )
rem  * on the "Font" tab, select "Lucida Console", 12 ( pixels high ), Bold
rem  * on the "Colors" tab, set the "Screen Text" to white
rem  * on the "Colors" tab, set the "Screen Background" to black
rem
rem You'll then see one row of data appear for each sample point and can use
rem the horizonal scroll bar at the bottom of the window to adjust the display left
rem and right.  Adjust the "Properties" settings further for your viewing and
rem monitor capabilities.
rem
rem
rem
rem *** AsciiDisplay ANALOG UNITS:
rem
rem AsciiDisplay always displays its analog channel values as counts.
rem Use DvmDisplay to see data in scaled units like volts.
rem
rem



rem Keep all title and path changes local ( ie restore on exit )

setlocal

set BATCHFILE=Ascii.bat

title SR "%BATCHFILE%" starting - ( Blast to AsciiDisplay )
path C:\SR\USBXCH\Exe;%PATH%



rem Announce the program start:

echo Running "%BATCHFILE%"
echo.



rem Set an environment variable with the INI filename.
rem
rem This single file is shared among all the utilities and has [sections]
rem for each.  %CD% indicates the current directory.
rem
rem

set INIFILE=%CD%\Ascii.ini




rem Check the INI syntax:
rem
rem Before executing check the INI syntax for each SR utility.
rem
rem   ... if there is an INI error fix it and try again ...
rem 
rem NOTE: >>>
rem
rem These syntax checks are done without /to clause on the utilities.  Because
rem of that the "nextprog" field on the INI reports will be reported as NULL.  See
rem below for the /to clause on each when the utilities are actually run.
rem 
rem 

set SR_RPT="SrRpt-All.rpt"

echo.
echo *** CHECKING INI syntax in "%INIFILE%" ...
echo.

echo Blast
Blast /syntax "%INIFILE%"               > %SR_RPT%
if ERRORLEVEL 1 ( goto :SyntaxError )

echo Pak2bin
Pak2Bin /syntax "%INIFILE%"             >> %SR_RPT%
if ERRORLEVEL 1 ( goto :SyntaxError )

echo AsciiDisplay
AsciiDisplay /syntax "%INIFILE%"        >> %SR_RPT%
if ERRORLEVEL 1 ( goto :SyntaxError )

goto :SyntaxGood

:SyntaxError
    echo. 
    echo     ... ERROR: bad ini syntax, see the respective *.rpt file for details
    echo.
    goto :Return

:SyntaxGood
    echo.
    echo Success, all INI syntax checks passed ...
    echo.




rem Check the pipeline utilities are not running already:
rem
rem Only one instance at a time is allowed, so check before executing.
rem
rem   ... if there is an error fix it and try again ...
rem 
rem 

echo.
echo *** CHECKING the utilities are not running already ...
echo.

call IsRunning.bat Blast              >> %SR_RPT%
if ERRORLEVEL 1 goto :MultipleError

call IsRunning.bat Pak2Bin            >> %SR_RPT%
if ERRORLEVEL 1 goto :MultipleError

call IsRunning.bat AsciiDisplay       >> %SR_RPT%
if ERRORLEVEL 1 goto :MultipleError

goto :MultipleGood

:MultipleError
    echo. 
    echo *** ERROR, failed running already check, quitting ...
    echo.
    goto :Return

:MultipleGood
    echo.
    echo *** SUCCESS, running already check passed ...
    echo.




rem Check for USBxCH driver and board presence.

echo.
echo *** CHECKING for USBxCH driver and board presence ( SrUsbXch0 ):

Presence.exe SrUsbXch0
if ERRORLEVEL 1 goto :Return



rem Clean up temporary report files.

del *.rpt




rem Start the pipeline chain in the background.
rem
rem NOTES:
rem
rem * Sampling rates, etc ... are all set in the single INI file
rem * "Start /b" is the Windows way of starting a process in the background
rem * AsciiDisplay.exe runs in the foreground, no /b on its "start"
rem
rem

echo.
echo *** STARTING the pipeline in the background ...

start /b           Blast.exe  "%INIFILE%"  /to Pak2Bin
start /b         Pak2Bin.exe  "%INIFILE%"  /to AsciiDisplay
start       AsciiDisplay.exe  "%INIFILE%"  /to NULL

echo.
echo *** The pipeline is now running in the background ... see *.rpt files for info
echo.



rem Return, telling the user what will happen.
rem
rem

:Return

echo.
echo.
echo "%BATCHFILE%": Return NOTE
echo.
echo If "%BATCHFILE%" was started by double clicking from Windows File Explorer,
echo this window will go away after "press any key to continue" ...
echo.
echo If "%BATCHFILE%" was started from a command prompt, it will return to the
echo command prompt.  Run tasklist etc to see processes running ...
echo.
pause
echo.

