@echo off

rem FILE: Dvm.bat  ( App - DVM )
rem COPYRIGHT: (c), Symmetric Research, 2014-2019
rem
rem This is an acquisition program.  It shows how to set up a pipeline
rem to bring acquired data all the way from the USBxCH board to a Digital 
rem Volt Meter style display.  It interpolates and saves the data on the way.
rem
rem "The program" is actually this batch file.  Five pipeline utilities are
rem used: ( Blast , Pak2Bin , Interp , Bin2Asc , DvmDisplay ).  Data moves from 
rem one utility to the next via intermediate named pipes.
rem
rem For viewing data as text or wiggly traces, see the batch files in the
rem "App - Ascii" and "App - Scope" directories respectively.
rem
rem See the "Examples" directory for pipelines aimed at specific applications.
rem 
rem Documentation for all of the pipeline utilities can be found in the "Docs"
rem directory.  Executables are in the "Exe" directory.
rem
rem The comments below have further info about the batch file constructs used.
rem
rem



rem Keep all title and path changes local ( ie restore on exit )

setlocal

set BATCHFILE=Dvm.bat

title SR "%BATCHFILE%" starting - ( Blast to DvmDisplay )
path C:\SR\USBXCH\Exe;%PATH%



rem Announce the program start:

echo Running "%BATCHFILE%"
echo.




rem Set an environment variable with the INI filename.
rem
rem This single INI is shared among all the utilities and has [sections] for
rem each.  %CD% indicates the current directory.
rem
rem NOTE: Referring to a batch argument with percent tilda like %~1 
rem       instead of %1 strips off any quotes.  The quotes can be added 
rem       later where they are needed.
rem       
rem       

set INIFILE=%~1
if "%INIFILE%" == "" (
        set INIFILE=%CD%\DvmSetup.ini
)



rem Check the INI syntax:
rem
rem Before executing check the INI syntax for each SR utility.
rem
rem   ... if there is an INI error fix it and try again ...
rem 
rem NOTE: >>>
rem
rem These syntax checks are done without /to clause on the utilities.  Because
rem of that the "nextprog" field on the INI reports will be reported as NULL.  See
rem below for the /to clause on each when the utilities are actually run.
rem 
rem 

set SR_RPT="SrRpt-All.rpt"

echo.
echo *** CHECKING INI syntax in "%INIFILE%" ...
echo.

echo Blast
Blast /syntax "%INIFILE%"               > %SR_RPT%
if ERRORLEVEL 1 ( goto :SyntaxError )

echo Pak2bin
Pak2Bin /syntax "%INIFILE%"             >> %SR_RPT%
if ERRORLEVEL 1 ( goto :SyntaxError )

echo Interp
Interp /syntax "%INIFILE%"              >> %SR_RPT%
if ERRORLEVEL 1 ( goto :SyntaxError )

echo Bin2Asc
Bin2Asc /syntax "%INIFILE%"             >> %SR_RPT%
if ERRORLEVEL 1 ( goto :SyntaxError )

echo DvmDisplay
DvmDisplay /syntax "%INIFILE%"          >> %SR_RPT%
if ERRORLEVEL 1 ( goto :SyntaxError )

goto :SyntaxGood


:SyntaxError
    echo. 
    echo     ... ERROR: bad ini syntax, see the respective *.rpt file for details
    echo. 
    goto :Return

:SyntaxGood
        echo.
    echo Success, all INI syntax checks passed ...
    echo.




rem Check the pipeline utilities are not running already:
rem
rem Only one instance at a time is allowed, so check before executing.
rem
rem   ... if there is an error fix it and try again ...
rem 
rem 

echo.
echo *** CHECKING the utilities are not running already ...
echo.

call IsRunning.bat Blast              >> %SR_RPT%
if ERRORLEVEL 1 goto :MultipleError

call IsRunning.bat Pak2Bin            >> %SR_RPT%
if ERRORLEVEL 1 goto :MultipleError

call IsRunning.bat Interp             >> %SR_RPT%
if ERRORLEVEL 1 goto :MultipleError

call IsRunning.bat Bin2Asc            >> %SR_RPT%
if ERRORLEVEL 1 goto :MultipleError

call IsRunning.bat DvmDisplay         >> %SR_RPT%
if ERRORLEVEL 1 goto :MultipleError

goto :MultipleGood

:MultipleError
    echo. 
    echo *** ERROR, failed running already check, quitting ...
    echo.
    goto :Return

:MultipleGood
    echo.
    echo *** SUCCESS, running already check passed ...
    echo.




rem Check for USBxCH driver and board presence.

echo.
echo *** CHECKING for USBxCH driver and board presence ( SrUsbXch0 ):

Presence.exe SrUsbXch0
if ERRORLEVEL 1 goto :Return



rem Clean up temporary report files.

del *.rpt




rem Create a YMDHMS data directory:
rem
rem YMDHMS = ( year, month, day, hour, minute, second).
rem
rem The SR YmdHms.exe utility creates a blank YMDHMS directory for storing 
rem data from the current acquisition run.  An actual directory name will be 
rem something like "2014-05-29-at-15-47-46" reflecting the current date 
rem and time.
rem
rem YmdHms.exe also writes the batch file YmdHmsSetEnv.bat for setting an
rem environment variable with the specific YMDHMS that can be referred to in 
rem the rest of this batch file.
rem
rem All processing utilities are run in the YMDHMS directory so any output
rem files are saved there.
rem
rem

echo.
echo *** CREATING the YMDHMS data directory for this run...

YmdHms  > nul
call YmdHmsSetEnv.bat

echo.
echo    success ... created: %SR_YMDHMS%
cd %SR_YMDHMS%




rem Start the pipeline chain in the background.
rem
rem NOTES:
rem
rem * Sampling rates, etc ... are all set in the single INI file
rem * "Start /b" is the Windows way of starting a process in the background
rem
rem

echo.
echo.
echo *** STARTING the pipeline in directory .\%SR_YMDHMS% ...

start /b        Blast.exe  "%INIFILE%"  /to Pak2Bin
start /b      Pak2Bin.exe  "%INIFILE%"  /to Interp
start /b       Interp.exe  "%INIFILE%"  /to Bin2Asc
start /b      Bin2Asc.exe  "%INIFILE%"  /to DvmDisplay
start /b   DvmDisplay.exe  "%INIFILE%"  /to NULL

echo.
echo *** The pipeline is now running in the background ... see *.rpt files for info
echo.



rem Return, telling the user what will happen.
rem
rem

:Return

echo.
echo.
echo "%BATCHFILE%": Return NOTE
echo.
echo If "%BATCHFILE%" was started by double clicking from Windows File Explorer,
echo this window will go away after "press any key to continue" ...
echo.
echo If "%BATCHFILE%" was started from a command prompt, it will return to the
echo command prompt.  Run tasklist etc to see processes running ...
echo.
pause
echo.

