@echo off

rem FILE: Compare.bat  ( Example Interpolating )
rem COPYRIGHT: (c), Symmetric Research, 2016
rem
rem This example sets up a pipeline to acquire data from a USBxCH board,
rem then save the data both before and after interpolating.  The five
rem pipeline utilities used are: ( Blast, Pak2Bin, Interp, Bin2Asc, 
rem ScopeDisplay ).  ScopeDisplay shows the interpolated data as it is 
rem acquired.  Use PlotCompare.bat to see the results with GnuPlot.
rem
rem



rem Save path to the current example directory.

set EXAMPLEDIR=%CD%


rem Keep all title and path changes local ( ie restore on exit )

setlocal

title SR COMPARE.BAT - ( Compare Interpolated Data )
path C:\SR\USBXCH\Exe;%PATH%



rem Announce the program start:

echo COMPARE.BAT: Starting ...
echo.



rem Make duplicate copies of the needed executables, but add
rem different suffixes so they can all run simultaneously without
rem pipe names colliding.
rem
rem

set EXEDIR=C:\SR\USBXCH\Exe

copy /y %EXEDIR%\Bin2Asc.exe %EXEDIR%\Bin2AscBefore.exe
copy /y %EXEDIR%\Bin2Asc.exe %EXEDIR%\Bin2AscAfter.exe



rem Set an environment variable with the INI file names.
rem
rem       

set INI_BEFORE=%EXAMPLEDIR%\Before.ini
set INI_AFTER=%EXAMPLEDIR%\After.ini



rem Create a YMDHMS data directory:
rem
rem YMDHMS = ( year, month, day, hour, minute, second).
rem
rem The SR YmdHms.exe utility creates a blank YMDHMS directory for storing data
rem from the current acquisition run.  An actual directory name will be 
rem something like "2014-05-29-at-15-47-46" reflecting the current date and 
rem time.
rem
rem YmdHms.exe also writes the batch file YmdHmsSetEnv.bat for setting an
rem environment variable with the specific YMDHMS that can be referred to in 
rem the rest of this batch file.
rem
rem All processing utilities are run in the YMDHMS directory so any output
rem files are saved there.
rem
rem

echo.
echo COMPARE.BAT: Creating the YMDHMS data directory for this run:

YmdHms  > nul
call YmdHmsSetEnv.bat

echo.
echo    success ... created: %SR_YMDHMS%
cd %SR_YMDHMS%




rem Check for USBxCH driver and board presence.

echo.
echo COMPARE.BAT: Checking for USBxCH presence ( SrUsbXch0 ):
echo.

Presence.exe SrUsbXch0

if ERRORLEVEL 1  goto :Return




rem Check the INI syntax:
rem
rem Before actually executing it is best to check the INI syntax.  If there is
rem a syntax error, this batch file terminates.  Fix the error and try again.
rem 
rem Note:
rem
rem These syntax checks are done without the /to clause on the utilities.  
rem Because of that the "nextprog" field on the INI reports will be reported 
rem as NULL.  See below for the /to clause on each when the utilities are 
rem actually run.
rem 
rem 

echo.
echo.
echo.
echo COMPARE.BAT: Checking INI syntax:
echo.

Blast /syntax "%INI_BEFORE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

Pak2Bin /syntax "%INI_BEFORE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

Bin2AscBefore /syntax "%INI_BEFORE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

Interp /syntax "%INI_AFTER%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

Bin2AscAfter /syntax "%INI_AFTER%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

ScopeDisplay /syntax "%INI_AFTER%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

goto :SyntaxGood


:SyntaxError
echo. 
echo COMPARE.BAT: ERROR, failed INI syntax check, quitting ...
echo. 
goto :Return

:SyntaxGood
echo.
echo COMPARE.BAT: SUCCESS, INI syntax check passed ...
echo.




rem Start the pipeline chain in the background.
rem
rem NOTES:
rem
rem * Sampling rates, etc ... are all set in the single INI file
rem * "Start /b" is the Windows way of starting a process in the background
rem
rem

echo.
echo.
echo COMPARE.BAT: Starting the pipeline in the background ...

start /b          Blast.exe  "%INI_BEFORE%"  /to Pak2Bin
start /b        Pak2Bin.exe  "%INI_BEFORE%"  /to Bin2AscBefore
start /b  Bin2AscBefore.exe  "%INI_BEFORE%"  /to Interp

start /b         Interp.exe  "%INI_AFTER%"   /to Bin2AscAfter
start /b   Bin2AscAfter.exe  "%INI_AFTER%"   /to ScopeDisplay
start /b   ScopeDisplay.exe  "%INI_AFTER%"   /to NULL

echo COMPARE.BAT: background pipeline started ... see *.rpt files for info
echo.



rem Return, telling the user what will happen.
rem
rem

:Return

echo.
echo.
echo COMPARE.BAT: Return NOTE
echo.
echo If Compare.bat was started by double clicking from Windows File Explorer,
echo this window will go away after "press any key to continue" ...
echo.
echo If Compare.bat was started from a command prompt, it will return to the
echo command prompt.  Run tasklist etc to see processes running ...
echo.
pause
echo.

