@echo off

rem FILE: Detect.bat  ( Example Pipeline - Triggering )
rem COPYRIGHT: (c), Symmetric Research, 2016
rem
rem This example sets up a pipeline to acquire data from a USBxCH board,
rem then detect and save events which exceed the specified amplitude threshold.
rem The five pipeline utilities used are: ( Blast, Pak2Bin, ScopeDisplay,
rem Trigger, Bin2Asc ).  ScopeDisplay shows all the data as it is acquired.
rem For viewing just the events, use ViewEvent.bat in this directory.
rem
rem If you want to use data packets from existing PAK files instead of ones
rem newly acquired by Blast, just comment in the two lines starting with the
rem words "rem PAK" and comment out the following Blast lines.  Be sure to
rem edit the Detect.ini file so it specifies the PAK files of interest.
rem
rem



rem Save path to the current example directory.

set EXAMPLEDIR=%CD%


rem Keep all title and path changes local ( ie restore on exit )

setlocal

title SR DETECT.BAT - ( Amplitude Trigger for Detecting Events )
path C:\SR\USBXCH\Exe;%PATH%



rem Announce the program start:

echo DETECT.BAT: Starting ...
echo.




rem Create a YMDHMS data directory:
rem
rem YMDHMS = ( year, month, day, hour, minute, second).
rem
rem The SR YmdHms.exe utility creates a blank YMDHMS directory for storing data
rem from the current acquisition run.  An actual directory name will be 
rem something like "2014-05-29-at-15-47-46" reflecting the current date and 
rem time.
rem
rem YmdHms.exe also writes the batch file YmdHmsSetEnv.bat for setting an
rem environment variable with the specific YMDHMS that can be referred to in 
rem the rest of this batch file.
rem
rem All processing utilities are run in the YMDHMS directory so any output
rem files are saved there.
rem
rem

echo.
echo DETECT.BAT: Creating the YMDHMS data directory for this run:

YmdHms  > nul
call YmdHmsSetEnv.bat

echo.
echo    success ... created: %SR_YMDHMS%
cd %SR_YMDHMS%




rem Check for USBxCH driver and board presence.

echo.
echo DETECT.BAT: Checking for USBxCH presence ( SrUsbXch0 ):
echo.

Presence.exe SrUsbXch0

if ERRORLEVEL 1  goto :Return




rem Set an environment variable with the INI file name.
rem
rem Take the name from the batch command line or use the default.  This
rem one file is shared among all of the pipeline utilities comprising 
rem Detect.bat and has [sections] for each utility.
rem
rem NOTE: Referring to a batch argument with percent tilda like %~1 
rem       instead of %1 strips off any quotes.  The quotes can be added 
rem       later where they are needed.
rem       
rem       

set INIFILE=%~1
if "%INIFILE%" == "" (
        set INIFILE=%EXAMPLEDIR%\Detect.ini
)



rem Check the INI syntax:
rem
rem Before actually executing it is best to check the INI syntax.  If there is
rem a syntax error, this batch file terminates.  Fix the error and try again.
rem 
rem Note:
rem
rem These syntax checks are done without the /to clause on the utilities.  
rem Because of that the "nextprog" field on the INI reports will be reported 
rem as NULL.  See below for the /to clause on each when the utilities are 
rem actually run.
rem 
rem 

echo.
echo.
echo.
echo DETECT.BAT: Checking INI syntax:
echo.

rem PAK PakFread /syntax "%INIFILE%"
Blast /syntax "%INIFILE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

Pak2Bin /syntax "%INIFILE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

Trigger /syntax "%INIFILE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

Bin2Asc /syntax "%INIFILE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

ScopeDisplay /syntax "%INIFILE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

goto :SyntaxGood


:SyntaxError
echo. 
echo DETECT.BAT: ERROR, failed INI syntax check, quitting ...
echo. 
goto :Return

:SyntaxGood
echo.
echo DETECT.BAT: SUCCESS, INI syntax check passed ...
echo.




rem Start the pipeline chain in the background.
rem
rem NOTES:
rem
rem * Sampling rates, etc ... are all set in the single INI file
rem * "Start /b" is the Windows way of starting a process in the background
rem
rem

echo.
echo.
echo DETECT.BAT: Starting the pipeline in the background ...

rem PAK start /b       PakFread.exe  "%INIFILE%"  /to Pak2Bin
start /b          Blast.exe  "%INIFILE%"  /to Pak2Bin
start /b        Pak2Bin.exe  "%INIFILE%"  /to Trigger
start /b        Trigger.exe  "%INIFILE%"  /to Bin2Asc
start /b        Bin2Asc.exe  "%INIFILE%"  /to ScopeDisplay
start /b   ScopeDisplay.exe  "%INIFILE%"  /to NULL

echo DETECT.BAT: background pipeline started ... see *.rpt files for info
echo.



rem Return, telling the user what will happen.
rem
rem

:Return

echo.
echo.
echo DETECT.BAT: Return NOTE
echo.
echo If Detect.bat was started by double clicking from Windows File Explorer,
echo this window will go away after "press any key to continue" ...
echo.
echo If Detect.bat was started from a command prompt, it will return to the
echo command prompt.  Run tasklist etc to see processes running ...
echo.
pause
echo.

