// FILE: Acquire.c (WINDOWS VERSION)
// COPYRIGHT: (c), Symmetric Research, 2014-2017
//
// Acquire is a USBxCH command line program to acquire data and save it
// in ASCII text files.  It does this by calling a chain of utility
// programs which are connected by pipes.
//


// Include files:

#include <stdio.h>
#include <stdlib.h>
#include <string.h>	// for strcmp
#include <ctype.h>      // for isdigit(), isxdigit()
#include <conio.h>      // kbhit, getch functions



// Program copyright and screen title:

char *Copyright =

          "\n\nAcquire.exe : Copyright (c), Symmetric Research, 2014-2017\n\n";

char *ScreenTitle =

      "USBxCH Acquire Output Acquisition Program: (c) SR 2014 : Rev {2017/02/17}\n\n";


// Defines and globals:

#define MAXSTR          256
#define REPORTFILENAME  "SrRpt-Acquire.rpt"

// ACQUIRE RUN TIME PARAMETERS:
//
// Both command line and fixed program parameters are saved here.  Access these
// values with statements like: PP.ProgramName.
//

struct AcquireParameters {

        char *SectionName;
        FILE *RptFp;
        char ProgramName[MAXSTR];

        } ProgramParameters = {  // initializer default values ...

                "Acquire",   // SectionName  <<< do not change
                NULL,        // RptFp, report file pointer
                // ... trailing fields set at run time

                };

#define PP  ProgramParameters  // PP = shorthand for ProgramParameters struct



// Prototypes:

void ProcessCommandLine( int argc, char *argv[] );
void RunSystemCommand( char *SysCommand, char *ErrMsg );
void PrintMsg( const char *msg );
void PrintError( const char *msg );
 int WaitForKeypress( void );



//------------------------------------------------------------------------------
// ROUTINE: main()
// PURPOSE: Acquire main entry point ...
//------------------------------------------------------------------------------
int main( int argc, char *argv[] ) {

        int  c;
        char *AcquireIni;
        char SysCommand[MAXSTR];


        // Process the command line arguments.

        ProcessCommandLine( argc, argv );  // fills in PP ...


        // Exec/spawn the Blast acquisition executable.

        AcquireIni = "Acquire.ini";

        sprintf( SysCommand, "\"start /b Blast %s /to Pak2Bin\"", AcquireIni );

        RunSystemCommand( SysCommand, "Failed to start Blast acquisition" );



        // Exec/spawn the Pak2Bin conversion executable.

        sprintf( SysCommand, "\"start /b Pak2Bin %s /to Bin2Asc\"", AcquireIni );

        RunSystemCommand( SysCommand, "Failed to start Pak2Bin conversion" );



        // Exec/spawn the Bin2Asc output executable.

        sprintf( SysCommand, "\"start /b Bin2Asc %s /to NULL\"", AcquireIni );

        RunSystemCommand( SysCommand, "Failed to start Bin2Asc output" );



        // Wait for keypress to abort ...

        PrintMsg( "\nIn another window, run tasklist to see background processes running\n\n\n" );

        PrintMsg( "Press q to kill background tasks ...\n" );

        c = WaitForKeypress();

        while ( (c != 'q') && (c != 'Q') ) {

                // != EOF -> there is a non quit keypress

                if ( c != EOF )
                        PrintMsg( "To abort press q ... \n" );

                c = WaitForKeypress(); // check for key
                }

        PrintMsg( "\n\n" );



        // Exec/spawn kill Bin2Asc.

        sprintf( SysCommand, "\"taskkill /f /im Bin2Asc.exe\"" );

        RunSystemCommand( SysCommand, "Failed to kill Bin2Asc" );


        // Exec/spawn kill Pak2Bin.

        sprintf( SysCommand, "\"taskkill /f /im Pak2Bin.exe\"" );

        RunSystemCommand( SysCommand, "Failed to kill Pak2Bin" );


        // Exec/spawn kill Blast.

        sprintf( SysCommand, "\"taskkill /f /im Blast.exe\"" );

        RunSystemCommand( SysCommand, "Failed to kill Blast" );



        PrintMsg( "SUCCESS: Acquire done.  See Out-* and Rpt*.rpt files for results.\n" );


        // Return from main:
        //
        // Linux *requires* main to be of type integer with return 0 for
        // success.  Windows doesn't care ...
        //
        //

        return( 0 );
}

//------------------------------------------------------------------------------
// ROUTINE: ProcessCommandLine
// PURPOSE: Gather the command line arguments.
//------------------------------------------------------------------------------
void ProcessCommandLine( int argc, char *argv[] ) {

        // Open report file.

        PP.RptFp = fopen( REPORTFILENAME, "w" );
        if ( PP.RptFp == NULL )
                PrintError( "Could not open report file\n" );


        // Announce program.

        PrintMsg( ScreenTitle );


        // Check for too many arguments.

        if ( argc > 2 )
                PrintError(
                           "Too many command line arguments.  Run with /? for help.\n" );

        // Process /? argument.

        if ( (argc == 2)  &&  (strcmp( argv[1], "/?" ) == 0) ) {
                printf( "Acquire.exe is a standalone USBxCH example which shows how to\n" );
                printf( "use the BIN pipeline utilities from inside a single executable.\n" );
                printf( "Input parameters are taken from Acquire.ini.\n\n" );
                printf( "Usage: Acquire.exe [/?]\n" );
                exit( 0 );
                }

        strcpy( PP.ProgramName, argv[0] );
}

//------------------------------------------------------------------------------
// ROUTINE: RunSystemCommand
// PURPOSE: Print a message to the console and report file.
//------------------------------------------------------------------------------
void RunSystemCommand( char *SysCommand, char *ErrMsg ) {

        char MsgBuffer[MAXSTR];

        if ( system( SysCommand ) != -1 ) {

                sprintf( MsgBuffer, "Running: %s\n", SysCommand );
                PrintMsg( MsgBuffer );
                }

        else // system() returned -1 ... error

                PrintError( ErrMsg );
}

//------------------------------------------------------------------------------
// ROUTINE: PrintMsg
// PURPOSE: Print a message to the console and report file making sure it comes
//          out immediately.
//------------------------------------------------------------------------------
void PrintMsg( const char *msg ) {

        // Display the message to the screen.

        printf( "%s", msg );
        fflush( stdout );


        // Append the message to the report file.

        if ( PP.RptFp ) {
                fprintf( PP.RptFp, "%s", msg );
                fflush( PP.RptFp );
                }
}

//------------------------------------------------------------------------------
// ROUTINE: PrintError
// PURPOSE: Exit after printing a message to the console.
//------------------------------------------------------------------------------
void PrintError( const char *msg ) {

        PrintMsg( msg ); // print the message
        exit( 2 );       // exit level 2
}

//------------------------------------------------------------------------------
// ROUTINE: WaitForKeypress
// PURPOSE: Wait for a keypress and return it.
//------------------------------------------------------------------------------
int WaitForKeypress( void ) {

        int c;

        // Get a character.

        c = _getch();

        // Compensate for ENTER and extended keys.

        if (c == 0xD) printf("\n");  // add newline if char was ENTER
        if (c == 0x00 || c == 0xE0 ) // deal with extended key (fn,->)
                c = _getch();        // c has scan code

        return( c );
}
