// FILE: CustomPipeLib.c (WINDOWS VERSION)
// COPYRIGHT: (c), Symmetric Research, 2016-2019
//
// Utility functions to access USBxCH BIN data from a named pipes in C.
//
// These are "easier to use" than the underlying Windows functions,
// providing an easier interface.  The same functions have corresponding
// equivalents in Linux with different underlying OS level functions.
//
//


// Include files:

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>            // for CreateNamedPipe etc
#include <time.h>               // for time_t, struct tm, gmtime etc

#include "CustomPipeLib.h"      // for prototypes, BIN, and other SR defines



// Defines and global variables:

FILE *RptFp = NULL;             // report file pointer

char MsgBuffer[MAXSTRBIG];      // for building message string



// Custom Pipe Lib Functions:
//
//

//------------------------------------------------------------------------------
// ROUTINE: CustomReportOpen
// PURPOSE: Open a report file.  The provided name will have SrRpt- as a prefix
//          and .rpt as a suffix so the name will be consistent with the
//          standard USBxCH report files.  It exits on failure.
//------------------------------------------------------------------------------
void CustomReportOpen( char *name ) {

        char RptName[MAXSTR];

        sprintf( RptName, "SrRpt-%s.rpt", name );

        RptFp = fopen( RptName, "a" );
        if ( !RptFp )
                exit( 2 );
}

//------------------------------------------------------------------------------
// ROUTINE: CustomReportPrint
// PURPOSE: Write provided message to the previously opened report file.
//------------------------------------------------------------------------------
void CustomReportPrint( char *msg ) {

        if ( RptFp ) {
                fprintf( RptFp, "%s", msg );
                fflush( RptFp );
                }
}

//------------------------------------------------------------------------------
// ROUTINE: CustomReportClose
// PURPOSE: Close the previously opened report file.
//------------------------------------------------------------------------------
void CustomReportClose( void ) {

        if ( RptFp ) {
                fclose( RptFp );
                RptFp = NULL;
                }
}

//------------------------------------------------------------------------------
// ROUTINE: CustomOpenInputPipe
// PURPOSE: Open a named pipe for input using the provided name.  Returns a
//          handle to the pipe for passing to other pipe functions.  If the
//          named pipe can not be opened, SR_BADHANDLE is returned.
//------------------------------------------------------------------------------
SRPIPEHANDLE CustomOpenInputPipe( char *InputPipeName ) {

        SRPIPEHANDLE hInputPipe;
        DWORD        AccessMode, ShareMode;


        // Initialize mode variables.
        
        AccessMode = GENERIC_READ | FILE_WRITE_ATTRIBUTES;
        ShareMode  = FILE_SHARE_READ | FILE_SHARE_WRITE;

        while ( 1 ) { 

                // Open the pipe for binary reading.

                hInputPipe = CreateFile(
                                   InputPipeName,
                                   AccessMode,   
                                   ShareMode,    
                                   NULL,                  // LPSECURITY_ATTRIBUTES
                                   OPEN_EXISTING,
                                   FILE_ATTRIBUTE_NORMAL, // DWORD
                                   NULL                   // SRPIPEHANDLE hTemplateFile
                                  );
                        
                // If successful, leave infinite loop.

                if ( hInputPipe != SR_BADHANDLE )
                        break;

                // Otherwise, wait and try again.

                CustomSleep( 500 );
                }
        
        sprintf( MsgBuffer, "Opened input pipe %s\n", InputPipeName );
        CustomReportPrint( MsgBuffer );

        return( hInputPipe );
}

//------------------------------------------------------------------------------
// ROUTINE: CustomReadInputPipe
// PURPOSE: Read one BIN point from the named input pipe using the provided
//          pipe handle.  Returns 1 on success and 0 otherwise.  The bytes read
//          in are checked to verify they contain the expected BIN point header.
//------------------------------------------------------------------------------
int CustomReadInputPipe( SRPIPEHANDLE hInputPipe, BIN *Pt ) {

        BOOL  Ok;
        int   Err;
        DWORD NumToRead, NumRead, NumRemain, BytesRead;
        char *BytePtr;


	// Error check.

	if ( hInputPipe == SR_BADHANDLE ) {
		CustomReportPrint( "Input pipe bad - can't read.\n" );
		return( 0 );
		}

	
        // Initialize variables.

        BytePtr   = (char *)Pt;
        NumToRead = NSIZE_BINDATA;
        NumRead   = 0;
        NumRemain = NumToRead - NumRead;


        // Loop around until all requested bytes have been read.
        
        while ( NumRead < NumToRead ) {

                Ok = ReadFile(
                              hInputPipe,  // SRPIPEHANDLE
                              BytePtr,     // LPVOID
                              NumRemain,
                              &BytesRead,
                              NULL         // LPOVERLAPPED
                             );
                Err = GetLastError();

                BytePtr   += BytesRead;
                NumRead   += BytesRead;
                NumRemain -= BytesRead;

                if ( !Ok ) {
                        sprintf( MsgBuffer,
                                 "Failed to read input pipe with system error %d\n",
                                 Err );
                        CustomReportPrint( MsgBuffer );
                        
                        FormatMessage(
                                      FORMAT_MESSAGE_FROM_SYSTEM,
                                      NULL,
                                      Err,
                                      0,
                                      (LPTSTR)&MsgBuffer,
                                      MAXSTR,
                                      NULL
                                     );
                        CustomReportPrint( MsgBuffer );
                        
                        return( 0 );
                        }
                } // end while


        // Verify we've got a BIN structure by checking its id.

        if ( Pt->Id == BIN_ID )
                return( 1 ); // success

        else {
                CustomReportPrint( "Invalid BIN structure read\n" );
                return( 0 );
                }
}

//------------------------------------------------------------------------------
// ROUTINE: CustomCloseInputPipe
// PURPOSE: Close the named input pipe using the provided pipe handle.
//------------------------------------------------------------------------------
void CustomCloseInputPipe( SRPIPEHANDLE hInputPipe ) {

        if ( hInputPipe != SR_BADHANDLE ) {
		CloseHandle( hInputPipe ); // close on own end
		hInputPipe = SR_BADHANDLE;
                CustomReportPrint( "Closed input pipe\n" );
                }
        else {
                CustomReportPrint( "Input pipe already closed\n" );
                }
}

//------------------------------------------------------------------------------
// ROUTINE: CustomOpenOutputPipe
// PURPOSE: Open a named pipe for output using the provided name.  Returns a
//          handle to the pipe for passing to other pipe functions.  If the
//          named pipe can not be opened, SR_BADHANDLE is returned.  This
//          function will wait forever for a pipe reader to connect before
//          returning.
//------------------------------------------------------------------------------
SRPIPEHANDLE CustomOpenOutputPipe( char *OutputPipeName ) {

        SRPIPEHANDLE hOutputPipe;
        DWORD        PipeSize, OpenMode, PipeMode;
        BOOL         Ok;


        // Initialize mode variables.

        OpenMode = PIPE_ACCESS_OUTBOUND | FILE_FLAG_FIRST_PIPE_INSTANCE;
        PipeMode = PIPE_TYPE_BYTE | PIPE_READMODE_BYTE | PIPE_WAIT;
        PipeSize = NSIZE_BINDATA * 10000 * 2; // sizeof(BIN) * max sample rate * 2 seconds


        // Prepare output pipe.

        hOutputPipe = CreateNamedPipe(
                                OutputPipeName,
                                OpenMode,
                                PipeMode,
                                1,          // DWORD nMaxInstances,
                                PipeSize,   // DWORD nOutBufferSize,
                                PipeSize,   // DWORD nInBufferSize,
                                0,          // DWORD nDefaultTimeOut, (ms)
                                NULL        // LPSECURITY_ATTRIBUTES
                                     );

        if ( hOutputPipe == SR_BADHANDLE )
                CustomErrorExit( "Failed to open output pipe\n" );


        // Wait until reader is connected before returning.

        Ok  = ConnectNamedPipe( hOutputPipe, NULL );

        if ( !Ok )
                CustomErrorExit( "Can't connect output pipe\n" );

        sprintf( MsgBuffer, "Opened output pipe %s\n", OutputPipeName );
        CustomReportPrint( MsgBuffer );

        return( hOutputPipe );
}

//------------------------------------------------------------------------------
// ROUTINE: CustomWriteOutputPipe
// PURPOSE: Write one BIN point to the named output pipe using the provided
//          pipe handle.  Returns 1 on success and 0 otherwise.
//------------------------------------------------------------------------------
int CustomWriteOutputPipe( SRPIPEHANDLE hOutputPipe, BIN *Pt ) {

        BOOL   Ok;
        int    Err;
        DWORD  NumToWrite, NumWritten, NumRemain, BytesWritten;
        char  *BytePtr;


	// Error check.

	if ( hOutputPipe == SR_BADHANDLE ) {
		CustomReportPrint( "Output pipe bad - can't write.\n" );
		return( 0 );
		}


        // Initialize variables.

        BytePtr      = (char *)Pt;
        NumToWrite   = NSIZE_BINDATA;
        NumWritten   = 0;
        NumRemain    = NumToWrite - NumWritten;


        // Write until all requested bytes written.

        while ( NumWritten < NumToWrite ) {

                Ok = WriteFile(
                               hOutputPipe,
                               BytePtr,       // LPCVOID lpBuffer,
                               NumRemain,
                               &BytesWritten,
                               NULL           // LPOVERLAPPED
                              );
                Err = GetLastError();

                BytePtr    += BytesWritten;
                NumWritten += BytesWritten;
                NumRemain  -= BytesWritten;

                if ( Err != ERROR_SUCCESS ) {
			CustomReportPrint( "Can't write to output pipe\n" );
			return( 0 );
			}

		} // end while

	return( 1 );
}

//------------------------------------------------------------------------------
// ROUTINE: CustomCloseOutputPipe
// PURPOSE: Close the named output pipe using the provided pipe handle.
//------------------------------------------------------------------------------
void CustomCloseOutputPipe( SRPIPEHANDLE hOutputPipe ) {
        
        if ( hOutputPipe != SR_BADHANDLE ) { 
                FlushFileBuffers( hOutputPipe );    // wait 'til client is done reading
                DisconnectNamedPipe( hOutputPipe ); // force close on client end
		CloseHandle( hOutputPipe );         // close on own end
		hOutputPipe = SR_BADHANDLE;
                CustomReportPrint( "Closed output pipe\n" );
                }
        else {
                CustomReportPrint( "Output pipe already closed\n" );
                }
}

//------------------------------------------------------------------------------
// ROUTINE: CustomErrorExit
// PURPOSE: Writes the provided message to the previously opened report file,
//          closes the report file, and exits.
//------------------------------------------------------------------------------
void CustomErrorExit( char *msg ) {

        CustomReportPrint( msg );
        CustomReportClose( );
        exit( 1 );
}

//------------------------------------------------------------------------------
// ROUTINE: CustomSleep
// PURPOSE: Sleeps for at least the provided number of milliseconds.
//------------------------------------------------------------------------------
void CustomSleep( int ms ) {

        Sleep( ms );
}

//------------------------------------------------------------------------------
// ROUTINE: CustomPrint64
// PURPOSE: This converts a 64-bit integer into a string.  It returns a pointer
//          to a static string buffer.  When printing TWO strings, copy the
//          first to another buffer before calling this function for the second
//          value !
//------------------------------------------------------------------------------
char *CustomPrint64( SRINT64 Value ) {

        static char StringBuffer[MAXSTR];

// In both 64-bit and 32-bit Windows, longs are 4 bytes so a 64 bit number 
// is referred to as a "long long int" which is printed with %lld.
//
        sprintf( StringBuffer, "%lld", Value ); // Windows 64 or 32 specifier

        return( StringBuffer );
}

//------------------------------------------------------------------------------
// ROUTINE: CustomExtractTime
// PURPOSE: This splits a 64 bit time such as the TimeStamp value from a BIN
//          point into human manageable pieces.  The YMDHMS pieces have the 
//          normal values (eg Year is 2019 and January is Month 1).  The 
//          unsigned integer seconds and nanoseconds are counted from Jan 1, 
//          1970.  It is ok to pass NULL for any values you are not interested
//          in.
//------------------------------------------------------------------------------
void CustomExtractTime( SRINT64 TimeStamp,
                        int *Year, int *Month, int *Day,
                        int *Hour, int *Minute, int *Second,
                        unsigned int *IntSec, unsigned int *NanoSec ) {

        unsigned int IntSec1970, NanoSec1970;
        time_t       ttTime;
        struct tm    tmTime;


        // Get the seconds since 1970 part as a time_t.
        // And use standard C function to split into YMDHMS.
        // Compute seconds and nanoseconds as unsigned integers.
        // When filling user variables, convert to normal year and month.

        ttTime = (time_t)( TimeStamp / (SRINT64)SR_NSPERSEC );
        tmTime = *gmtime( &ttTime );

        IntSec1970  = (unsigned int)( ttTime );
        NanoSec1970 = (unsigned int)( TimeStamp - 
                                    ((SRINT64)IntSec1970 * (SRINT64)SR_NSPERSEC) );

        if ( Year )     *Year    = tmTime.tm_year + 1900;
        if ( Month )    *Month   = tmTime.tm_mon + 1;
        if ( Day )      *Day     = tmTime.tm_mday;
        if ( Hour )     *Hour    = tmTime.tm_hour;
        if ( Minute )   *Minute  = tmTime.tm_min;
        if ( Second )   *Second  = tmTime.tm_sec;
        if ( IntSec )   *IntSec  = IntSec1970;
        if ( NanoSec )  *NanoSec = NanoSec1970;
}

//------------------------------------------------------------------------------
// ROUTINE: CustomExtractDigital
// PURPOSE: This takes the packed DigitalAll value from a BIN point and extracts
//          the 4 digital inputs into separate integer variables.  It is ok to
//          pass NULL for any values you are not interested in.
//------------------------------------------------------------------------------
void CustomExtractDigital( int DigitalAll,
                           int *Dig0, int *Dig1, int *Dig2, int *Dig3 ) {

        if ( Dig0 )  *Dig0 = (DigitalAll >> 0) & 0x01;
        if ( Dig1 )  *Dig1 = (DigitalAll >> 1) & 0x01;
        if ( Dig2 )  *Dig2 = (DigitalAll >> 2) & 0x01;
        if ( Dig3 )  *Dig3 = (DigitalAll >> 3) & 0x01;
}

//------------------------------------------------------------------------------
// ROUTINE: CustomExtractDegrees
// PURPOSE: This takes the DegC2 (degrees C times 2) value from a BIN point and
//          extracts the true degrees Centigrade and Farenheit.  The degrees C
//          are accurate to 0.5 degrees and the degrees F are accurate to 1
//          degree.  It is ok to pass NULL for any value you are not interested
//          in.
//------------------------------------------------------------------------------
void CustomExtractDegrees( int DegC2, double *DegreesC, double *DegreesF ) {

        int    intF;
        double DegC, DegF;

        // Compute true degrees Centigrade and Farenheit.

        DegC = (double)DegC2 / 2.0;
        DegF = (DegC * 1.8) + 32.0;

        // Round Farenheit degrees to nearest whole number.

        intF = (int)(DegF + 0.5);
        DegF = (double)intF;

        // Fill user variables if passed.
        
        if ( DegreesC )   *DegreesC = DegC;
        if ( DegreesF )   *DegreesF = DegF;
}

//------------------------------------------------------------------------------
// ROUTINE: CustomExtractVoltsGood
// PURPOSE: This takes the packed VoltageGood value from a BIN point and 
//          extracts the power history value.  A value of 1 means power has
//          been good, 0 otherwise.
//------------------------------------------------------------------------------
void CustomExtractVoltsGood( int VoltageGood, int *Vgood ) {

        if ( Vgood )  *Vgood = (VoltageGood >> 1) & 0x0001;
}

