// FILE: RequestSleep.c (WINDOWS VERSION)
// COPYRIGHT: (c), Symmetric Research, 2010-2016
//
// RequestSleep is a small helper program that can be easily called from
// a batch file such as SleepWake.bat.  It puts a Windows PC to sleep
// for the specified number of seconds and automatically wakes up and
// exits after those seconds have passed.  It takes two command line
// arguments.  The first specifies the number of seconds to sleep and
// the second specifies whether or not the display should be enabled on
// wakeup.
//

#include <windows.h>
#include <powrprof.h>  // for suspend and wake functions

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>



#define WAKE_TIMER  "WakeTimer"

#define DISPLAY_OFF  0
#define DISPLAY_ON   1

#define USPERSEC     1000000

 int SuspendAndWake( int Seconds, int DisplayOn );
void RestoreDisplay( void );
 int TimerValue( HANDLE TimerHandle, int MicroSeconds );

void main( int argc, char **argv ) {

        int Nok, Nsec, Ndisp, ShowUsage;

        // Process command line.

        ShowUsage = 1;
        if ( argc == 3 ) {                                              // if right # args
                Nok = sscanf( argv[1], "%d", &Nsec );                   // get 1st arg
                if ( Nok == 1  &&  Nsec >= 0 ) {                        // if ok
                        Nok = sscanf( argv[2], "%d", &Ndisp );          // get 2nd arg
                        if ( Nok == 1  &&  (Ndisp == 0 || Ndisp == 1) ) // if ok
                                ShowUsage = 0;                          // don't show usage msg
                        }
                }
        if ( ShowUsage == 1 ) {
                printf( "Usage: RequestSleep N D\n\n" );
                printf( "  Where\n" );
                printf( "         N is the number of seconds to sleep before\n" );
                printf( "              automatic wake up.  0 means no wakeup.\n" );
                printf( "         D is the display state on wakeup.  1 for on, 0 for off.\n" );
                exit( 1 );
                }


        // Request timed sleep with auto wake.
        
        if ( SuspendAndWake( Nsec, Ndisp ) )
                exit( 0 );
        else {
                printf( "RequestSleep failed suspend and wake call\n" );
                exit( 1 );
                }
}

//------------------------------------------------------------------------------
// ROUTINE: SuspendAndWake
// PURPOSE: Suspend the system by putting it into standby mode.  Optionally
//          set a timer to wake the system in the specified number of seconds.
//          Also specify whether or not to turn the display on when waking.
//          Returns 1 for success, 0 on error.
//------------------------------------------------------------------------------
int SuspendAndWake( int Seconds, int DisplayOn ) {

        HANDLE  TimerHandle;
        int     Result, MicroSeconds;
        BOOLEAN Ok;
        

        // If time requested, open and set wake timer.

        if ( Seconds > 0 ) {

                TimerHandle = CreateWaitableTimer(
                                  NULL,         // Default security attributes
                                  TRUE,         // Manual reset stays signaled until set
                                  WAKE_TIMER ); // Name
                if ( TimerHandle == NULL )
                        return( 0 );

                MicroSeconds = Seconds * USPERSEC;
                
                Result = TimerValue(  TimerHandle, MicroSeconds );
                if ( Result == 0 )
                        return( 0 );
                }



        // Put system into standby.

        Ok = SetSuspendState(
                             FALSE, // Suspend not Hibernate
                             FALSE, // Notify apps before suspend (not forced)
                             FALSE  // Allow waking not DisableWakeEvent
                            );
        if ( !Ok ) {
                CloseHandle( TimerHandle );
                return( 0 );
                }
                


        // Wait for the timer.

        if ( Seconds > 0 ) {

                if ( WaitForSingleObject( TimerHandle, INFINITE ) == WAIT_OBJECT_0 )
                        Result = 1;
                else
                        Result = 0;

                // Indicate that we require the display.

                if ( DisplayOn  || ( Result == 0 ) )
                        RestoreDisplay( );

                CloseHandle( TimerHandle );
                
                if ( Result == 0 )
                        return( 0 );

                }


        return( 1 );
}

//------------------------------------------------------------------------------
// ROUTINE: RestoreDisplay
// PURPOSE: Lets the OS know the screen display is required and should be
//          restored.
//------------------------------------------------------------------------------
void RestoreDisplay( void ) {

        static EXECUTION_STATE OldState;
        
        OldState = SetThreadExecutionState( ES_DISPLAY_REQUIRED );
}

//------------------------------------------------------------------------------
// ROUTINE: TimerValue
// PURPOSE: Assigns the wait time in microseconds to the timer associated with 
//          the provided handle.  Returns 1 for success, 0 on error.
//------------------------------------------------------------------------------
int TimerValue( HANDLE TimerHandle, int MicroSeconds ) {

        int            Err;
        BOOLEAN        Ok;
        LARGE_INTEGER  liDueTime;

        
        // Set wait time while ensuring calculations happen as LONGLONGs
        // to avoid overflow on intermediate ints.

        liDueTime.QuadPart  = MicroSeconds;  // number of microseconds requested
        liDueTime.QuadPart *= -10;           // * # 100 ns intervals/1 microsecond
                                             // - for relative time
        
        // Set length of timer.
        
        Ok = SetWaitableTimer( TimerHandle,// timer handle
                               &liDueTime, // when due
                               0,          // period
                               NULL,       // completion routine
                               NULL,       // arg to completion routine
                               TRUE );     // resume (auto wake) system
                
        Err = GetLastError();
        if ( !Ok || ( Err == ERROR_NOT_SUPPORTED ) )
                return( 0 );
        else
                return( 1 );
}

