@echo off

rem FILE: BlastToSeed.bat  ( Example Seismic App mini-SEED )
rem COPYRIGHT: (c), Symmetric Research, 2013-2018
rem
rem This is a demo program.  It shows how to set up a pipeline to bring data
rem all the way from the USBxCH board to mini-SEED files.
rem
rem "The program" is actually this batch file.  Four pipeline utilities are
rem used: ( Blast , Pak2Bin , Interp, Bin2Seed ).  Data moves from one utility 
rem to the next via intermediate named pipes.
rem
rem See the batch files in the "App - Ascii", "App - DVM" and "App - Scope"
rem directories for further info about the batch file constructs used and
rem for more complicated examples.
rem
rem Documentation for all of the pipeline utilities can be found in the "Docs"
rem directory.  Executables are in the "Exe" directory.
rem
rem Note: Bin2Seed MUST be the last utility in the chain.
rem
rem



rem Save path to the current example directory.

set EXAMPLEDIR=%CD%


rem Also save a path to a directory for the data.
rem When used with SeisComp, it must NOT contain any spaces.

rem set SEEDDIR=%EXAMPLEDIR%
set SEEDDIR=C:\SR\USBXCH\Examples\SeedData


rem Set title and path, keeping these changes local ( ie restore on exit )

setlocal

title SR BLASTTOSEED.BAT - ( Acquire mini-SEED files )
path C:\SR\USBXCH\Exe;%EXAMPLEDIR%;%PATH%



rem Announce the program start:

echo BLASTTOSEED.BAT: Starting ...
echo.




rem Go to a SeedData directory, creating it if needed:
rem

if exist %SEEDDIR% (
        echo
        echo BLASTTOSEED.BAT: SeedData directory already exists:
) else (
        echo
        echo BLASTTOSEED.BAT: Creating the SeedData directory for this run:

        mkdir %SEEDDIR%

        echo
        echo "   success ... created: %SEEDDIR%"
)

cd %SEEDDIR%


rem Optionally create a YMDHMS data directory:
rem
rem YMDHMS = ( year, month, day, hour, minute, second).
rem
rem The SR YmdHms.exe utility creates a blank YMDHMS directory for storing data
rem from the current acquisition run.  An actual directory name will be something
rem like "2014-05-29-at-15-47-46" reflecting the current date and time.
rem
rem YmdHms.exe also writes the batch file YmdHmsSetEnv.bat for setting an
rem environment variable with the specific YMDHMS that can be referred to in the
rem rest of this batch file.
rem
rem All processing utilities are run in the YMDHMS directory so any output
rem files are saved there.
rem
rem

rem Move the comment characters between these two lines to control whether
rem a YMDHMS subdirectory is created (Y) or not (N).

rem set MAKE_YMDHMS=Y
set MAKE_YMDHMS=N


if  "%MAKE_YMDHMS%" == "Y" (

        echo.
        echo BLASTTOSEED.BAT: Creating the YMDHMS data directory for this run:

        YmdHms  > nul
        call YmdHmsSetEnv.bat

        echo.
        echo    success ... created: %SR_YMDHMS%
        cd %SR_YMDHMS%
)

set DATADIR=%CD%




rem Check for USBxCH driver and board presence.

echo.
echo BLASTTOSEED.BAT: Checking for USBxCH presence ( SrUsbXch0 ):
echo.

Presence.exe SrUsbXch0

if ERRORLEVEL 1  goto :Return



rem Set an environment variable with the INI filename.
rem
rem The single INI file is shared among all of the standard pipeline utilities 
rem comprising BlastToSeed.bat and has a [section] for each utility.  The
rem custom Bin2Seed utility takes its parameters from the CFG file which has a
rem fixed format.
rem
rem

set INIFILE=%EXAMPLEDIR%\BlastToSeed.ini
set CONFIG=%EXAMPLEDIR%\Bin2Seed.cfg



rem Check the INI syntax:
rem
rem Before actually executing it is best to check the INI syntax.  If there is
rem a syntax error, this batch file terminates.  Fix the error and try again.
rem 
rem Note:
rem
rem These syntax checks are done without /to clause on the utilities.  Because
rem of that the "nextprog" field on the INI reports will be reported as NULL.  See
rem below for the /to clause on each when the utilities are actually run.
rem 
rem 

echo.
echo.
echo.
echo BLASTTOSEED.BAT: Checking INI syntax:
echo.

Blast /syntax "%INIFILE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

Pak2Bin /syntax "%INIFILE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

Interp /syntax "%INIFILE%"
if ERRORLEVEL 1 goto :SyntaxError
echo.

goto :SyntaxGood

:SyntaxError
    echo. 
    echo BLASTTOSEED.BAT: ERROR, failed INI syntax check, quitting ...
    echo.
    goto :Return

:SyntaxGood
    echo.
    echo BLASTTOSEED.BAT: SUCCESS, INI syntax checks passed ...
    echo.




rem Start the pipeline chain in the background.
rem
rem NOTES:
rem
rem * A single INI file is shared between all four utilities
rem * Sampling rates, etc ... are all set in the INI file BlastToSeed.ini
rem * The INI filename is specified once in the env variable INIFILE
rem
rem * "Start /b" is the Windows way of starting a process in the background
rem
rem
rem

echo.
echo.
echo BLASTTOSEED.BAT: Starting the pipeline in the background ...

start /b           Blast.exe  "%INIFILE%"  /to Pak2Bin
start /b         Pak2Bin.exe  "%INIFILE%"  /to Interp
start /b          Interp.exe  "%INIFILE%"  /to Bin2Seed
start /b        Bin2Seed.exe  "%CONFIG%"

echo BLASTTOSEED.BAT: background pipeline started ... see *.rpt files for info
echo.



rem Return, advising the user what will happen.
rem
rem

:Return

echo.
echo.
echo BLASTTOSEED.BAT: ***** Data is in: %DATADIR%
echo.
echo.

echo BLASTTOSEED.BAT: Return NOTE
echo.
echo If BlastToSeed.bat was started by double clicking from Windows File Explorer,
echo this window will go away after "press any key to continue" ...
echo.
echo If BlastToSeed.bat was started from a command prompt, it will return to the
echo command prompt.  Run tasklist etc to see processes running ...
echo.
pause
echo.

