// FILE: Bin2Seed.c
// COPYRIGHT: (c), Symmetric Research, 2011-2018
//
// Bin2Seed converts column binary output from Pak2Bin or Interp into 
// one miniSEED file per channel.
//
// The output SEED data is written to disk.  The input BIN data is 
// transferred using named pipes.  So no large BIN files are built
// up on disk.  Use the BinTools helper programs to save intermediate
// BIN files if needed.
//
//


// Include files:

#include <stdio.h>
#include <stdlib.h>
#include <string.h>	   // for strcmp, strncpy
#include <ctype.h>         // for tolower(), etc
#include <time.h>          // for time, localtime, strftime, tm*, etc

#include "CustomPipeLib.h" // for prototypes, BIN, and other SR defines

#if defined( SROS_WINDOWS )
#define SR_DIRSLASH '\\'
#define SR_MKDIR_OPTS ""
#elif defined( SROS_LINUX )
#define SR_DIRSLASH '/'
#define SR_MKDIR_OPTS "-p"
#include <errno.h>
#endif  // SROS_xxxxx



// Program copyright and screen title:

char *Copyright =

        "\n\nBin2Seed.exe : Copyright (c), Symmetric Research, 2011-2018\n\n";

char *ScreenTitle =

                "USBxCH Bin2Seed : (c) SR 2011 : Rev {2018/12/31}\n\n";





// Defines:

#define PROGRAMNAME   "Bin2Seed"

#define MAXFILENAME        1024
#define MAXSTR              256  // < generic string length
#define MAXCHANNELS           8  // maximum number of analog channels
#define MAXNAMENET            2  // max char in net name, eg SR
#define MAXNAMESTA            5  // max char in station name, eg SRHQ
#define MAXNAMELOC            2  // max char in sensor location/input name, eg C0
#define MAXNAMECMP            3  // max char in channel/component name, eg HNZ

#define NSPERSEC     1000000000  // nanoseconds / second
#define NSPERUS            1000  // nanoseconds / microsecond

#define MIN_GOOD_TIME 1454695792 // minimum seconds to consider time is ok

#define MSEED_NSAMPLES      109  // (MSEED_PACKET_SIZE -
                                 //  sizeof fixed hdr (48) -
                                 //  sizeof blockette 100 (12) -
                                 //  sizeof blockette 1001 (8) -
                                 //  sizeof blockette 1000 (8) ) /
                                 //  sizeof data sample (4)

#define CHAN_TYPE       'D'


// Naming convention defines:

#define NAMESTYLE_NONE  0       // not set yet
#define NAMESTYLE_SDS   1       // year/net/sta/chan.type/net.sta.loc.chan.type.year.day
#define NAMESTYLE_BUD   2       // net/sta/sta.net.loc.chan.year.day


// Wait for good time defines:

#define WAITFORGOODTIME_NONE 0  // not set yet
#define WAITFORGOODTIME_NO   1  // don't wait
#define WAITFORGOODTIME_YES  2  // wait for valid time before sending data


// SEED data type defines:

#define SEEDFLOAT float            // 4 bytes
#define SEEDLONG  int              // 4 bytes
#define SEEDWORD  short            // 2 bytes
#define SEEDUWORD unsigned short   // 2 bytes
#define SEEDBYTE  char             // 1 byte
#define SEEDUBYTE unsigned char    // 1 byte

struct BTIME; // for forward reference


// Global variables:

int   Nchannels;        // number output channels, must be 4 or 8 (default)

int   CurrentPoint;     // number of current point since start
int   FileNumber;       // sequence count of current file, starts at 1
int   SampleCount;      // number of sample within current file
int   PrecisionSec;     // uSec beyond precision of standard blockette time
float Sps;              // samples per second (Hz)
float Speriod;          // sample period (sec)

SRPIPEHANDLE hInputPipe;     // handle of input pipe
char  InputPipeName[MAXSTR]; // name of input pipe
char  MsgBuffer[MAXSTR];     // string for temporary messages
BIN   Pts[MSEED_NSAMPLES];   // data points

int   WaitForGoodTime;       // yes means wait to output data
int   NameStyle;             // SDS or BUD style naming
char  NameStyleName[4];      // name of layout style "SDS" or "BUD"

char  ChNameDir[MAXCHANNELS][MAXSTR];       // per channel directory name
char  ChNameFil[MAXCHANNELS][MAXSTR];       // per channel file name

char  ChNameNet[MAXCHANNELS][MAXNAMENET+1]; // per channel net name
char  ChNameSta[MAXCHANNELS][MAXNAMESTA+1]; // per channel station name
char  ChNameLoc[MAXCHANNELS][MAXNAMELOC+1]; // per channel location name
char  ChNameCmp[MAXCHANNELS][MAXNAMECMP+1]; // per channel component name




// Prototypes:

  void ProcessCommandLine( int argc, char *argv[] );
  void ReadConfig( char *CfgFileName );
  void Usage( void );
   int TimeIsGood( SRINT64 PointTime );
  void ComputeInputRate( void );
  void FinalOutput( void );
   int CleanExit( int ExitCode );
  void LogMessage( char *Msg, int Err );

   int MakeBtime( int IntSec1970, int NanoSec1970, struct BTIME *SampleBtime );
 short SwapInt2( short DataIn );
   int SwapInt4( int DataIn );
 float SwapFloat( float DataIn );
   int FillChars( int NumChar, char *Src, char *Dest );
  void MiniSeedOpen( int FirstOpen );
  void MiniSeedClose( void );
  void MiniSeedOutputTitle( SRINT64 Time64 );
  void MiniSeedOutputData( int NumPoints );
  void MiniSeedOutputPad( int NumPoints );
 

//------------------------------------------------------------------------------
// ROUTINE: main()
// PURPOSE: Main entry point for Bin2Seed ... convert BIN files to miniSEED.
//------------------------------------------------------------------------------
int main( int argc, char *argv[] ) {

        int rateComputed;

        // Initialize some starting values.

        rateComputed = 0;
        CurrentPoint = 0;
        FileNumber   = 1;
        SampleCount  = 0;
        hInputPipe   = SR_BADHANDLE;

        
        // Open the report file and announce the program.

        CustomReportOpen( PROGRAMNAME );
        CustomReportPrint( ScreenTitle );
        CustomReportPrint( "Bin2Seed starting\n" );


        // Process the command line arguments and configuration file.

        ProcessCommandLine( argc, argv );


        // Form name and open the input pipe.

        sprintf( InputPipeName, "%s%s", REQUIRED_PIPE_PREFIX, PROGRAMNAME );
        hInputPipe  = CustomOpenInputPipe( InputPipeName );

          // >> careful programmers might add error checking to confirm the
          // >> input pipe was successfully opened ...



        // Create miniSEED files.

        MiniSeedOpen( 1 ); // open for write
        MiniSeedClose( );


        CustomReportPrint( "Begin converting to SEED ...\n" );
        

        // While there is data, read the input and write the output ...

	while ( CustomReadInputPipe( hInputPipe, &Pts[SampleCount] ) ) {

                // Check that time is good before starting to count samples.

                if ( TimeIsGood( Pts[SampleCount].TimeStamp ) )
                        SampleCount++;
                else
                        SampleCount = 0;


                // Output once we have enough points to fill a file.

                if ( SampleCount == MSEED_NSAMPLES ) {

                        if ( !rateComputed ) {          // get header info the
                                ComputeInputRate( );    // first time through
                                rateComputed = 1;
                                }

                        // Output points to SEED file.

                        MiniSeedOpen( 0 ); // open for append
                        MiniSeedOutputTitle( Pts[0].TimeStamp );
                        MiniSeedOutputData( MSEED_NSAMPLES );
                        MiniSeedClose( );

                        // Show progress in report only.

                        sprintf( MsgBuffer, "Processed point %d\n", CurrentPoint );
                        CustomReportPrint( MsgBuffer );

                        SampleCount = 0; // begin collecting points for next file
                        }

                } // end while 

        // Output remaining points and any required padding to miniSeed files.
        
        FinalOutput( ); // closes output files
        

	// Close the input pipe.

	CustomCloseInputPipe( hInputPipe );
        

        // Announce done and close report file.

        CustomReportPrint( "SUCCESS: Stopping SEED conversion.\n" );
        CustomReportClose(  );
        
        
        // Return from main:
        //
        // Linux *requires* main to be of type integer with return 0 for
        // success.  Windows doesn't care ...
        //
        //

        exit( 0 );
}

//------------------------------------------------------------------------------
// ROUTINE: ProcessCommandLine
// PURPOSE: Get config file name from the command line, then read it.
//------------------------------------------------------------------------------
void ProcessCommandLine( int argc, char *argv[] ) {

        char CfgFileName[MAXSTR];


        // Process the command line for one required variable.

        if ( argc != 2 )
                Usage( );

        sprintf( CfgFileName, "%s", argv[1] );


        // Read configuration file.

        ReadConfig( CfgFileName );
}

//------------------------------------------------------------------------------
// ROUTINE: ReadConfig
// PURPOSE: Read user provided values from the provided configuration file.
//------------------------------------------------------------------------------
void ReadConfig( char *CfgFileName ) {

        int   i, j, ns;
        FILE *CfgFp;
        char  *str, NameNet[20], NameSta[20], NameLoc[20], NameCmp[20];
        char  WaitString[20];


        // Set variable defaults.

        Nchannels = 0;                          // # of chan is < MAXCHANNELS
        NameStyle = NAMESTYLE_NONE;             // name style is SDS or BUD
        WaitForGoodTime = WAITFORGOODTIME_NONE; // wait before sending data

        for ( i = 0 ; i < MAXCHANNELS ; i++ ) {
                sprintf( ChNameNet[i], "SR" );
                sprintf( ChNameSta[i], "SRHQ" );
                sprintf( ChNameLoc[i], "C%1d", i );
                sprintf( ChNameCmp[i], "HNZ" );
                }


        // Open provided config file.
        
        CfgFp = fopen( CfgFileName, "r" );
        
        if ( !CfgFp ) {
                sprintf( MsgBuffer, "unable to open cfg file %s\n", CfgFileName );
                LogMessage( MsgBuffer, 1 );
                }


        // Skip six lines (title + 4 header + blank).

        for ( i = 0 ; i < 6 ; i++ ) {
                str = fgets( MsgBuffer, MAXSTR, CfgFp );
                }


        // Read and verify NCHANNELS keyword.
                
        ns = fscanf( CfgFp, "NCHANNELS = %d\n\n", &Nchannels );
        if ( ( ns != 1 ) || ( Nchannels <= 0 ) || ( Nchannels >= MAXCHANNELS ) ) {
                fclose( CfgFp );
                sprintf( MsgBuffer, "Bad NCHANNELS, must be > 0 and < %d\n", MAXCHANNELS );
                LogMessage( MsgBuffer, 1 );
                }
                
        
        // Read and verify NAMESTYLE keyword.

        ns = fscanf( CfgFp, "NAMESTYLE = %s\n\n", NameStyleName );
        
        if ( ns == 1 ) {
                if ( strcmp( NameStyleName, "SDS" ) == 0 )
                        NameStyle = NAMESTYLE_SDS;
                if ( strcmp( NameStyleName, "BUD" ) == 0 )
                        NameStyle = NAMESTYLE_BUD;
                }
        if ( (ns != 1) ||
             (NameStyle != NAMESTYLE_SDS) &&
             (NameStyle != NAMESTYLE_BUD) ) {
                fclose( CfgFp );
                LogMessage( "Bad NAMESTYLE, must be SDS or BUD\n", 1 );
                }


        // Read and verify WAITFORGOODTIME keyword.

        ns = fscanf( CfgFp, "WAITFORGOODTIME = %s\n\n", WaitString );
        
        if ( ns == 1 ) {
                if ( strcmp( WaitString, "YES" ) == 0 )
                        WaitForGoodTime = WAITFORGOODTIME_YES;
                if ( strcmp( WaitString, "NO" ) == 0 )
                        WaitForGoodTime = WAITFORGOODTIME_NO;
                }
        if ( (ns != 1) ||
             (WaitForGoodTime != WAITFORGOODTIME_YES) &&
             (WaitForGoodTime != WAITFORGOODTIME_NO) ) {
                fclose( CfgFp );
                LogMessage( "Bad WAITFORGOODTIME, must be YES or NO\n", 1 );
                }


        // Read and verify and CHANNEL keywords.

        // >> careful programmers would protect against buffer overruns here...

        for ( i = 0 ; i < MAXCHANNELS ; i++ ) {
                ns = fscanf( CfgFp, "CHANNEL %d = %s %s %s %s\n",
                             &j, NameNet, NameSta, NameLoc, NameCmp );
                if ( (ns == 5) && (j >= 0) && (j < MAXCHANNELS) ) {
                          strncpy( ChNameNet[j], NameNet, MAXNAMENET+1 );
                          strncpy( ChNameSta[j], NameSta, MAXNAMESTA+1 );
                          strncpy( ChNameLoc[j], NameLoc, MAXNAMELOC+1 );
                          strncpy( ChNameCmp[j], NameCmp, MAXNAMECMP+1 );
                        }
                else {
                        fclose( CfgFp );
                        sprintf( MsgBuffer, "Bad CHANNEL %d, must be 0 to %d and Net Sta Loc Cmp\n",
                                 j, MAXCHANNELS );
                        LogMessage( MsgBuffer, 1 );
                        }
                }


        // Close configuration file.

        if ( fclose( CfgFp ) == EOF ) {
                sprintf( MsgBuffer, "unable to close cfg file\n" );
                LogMessage( MsgBuffer, 1 );
                }

        CfgFp = NULL;
}

//------------------------------------------------------------------------------
// ROUTINE: Usage
// PURPOSE: Show usage message to screen and save in report.
//------------------------------------------------------------------------------
void Usage( void ) {

        sprintf( MsgBuffer, "Usage: Bin2Seed CfgName\n\n" );
        LogMessage( MsgBuffer, 0 );
        
        sprintf( MsgBuffer, "  where CfgName is the configuration file name\n\n" );
        LogMessage( MsgBuffer, 0 );

        sprintf( MsgBuffer, "Example: Bin2Seed Bin2Seed.cfg\n" );
        LogMessage( MsgBuffer, 0 );
        
        CleanExit( 0 );
}

//------------------------------------------------------------------------------
// ROUTINE: TimeIsGood
// PURPOSE: Check whether the time value is reasonable.  Return 1 when it is ok
//          to start sending data and 0 before then.  Because the time is
//          written once at the start of the SEED file, it is important that the
//          time info is valid.  When using GPS time it can take at least 2
//          seconds to get a good time value.  So we don't want to write any
//          files with data until then.  But timing mode G0 starts from time 0
//          so an ini parameter WaitForGoodTime is provided to allow ignoring
//          this check.
//------------------------------------------------------------------------------
int TimeIsGood( SRINT64 PointTime ) {

 static int        BadTimeMsgSent  = 0;
 static int        GoodTimeMsgSent = 0;
        int        thisSec;
        char      *timestr;
        time_t     ltime;
        struct tm *currtime;


        // Once good time has been found, we don't need to check any more.
 
        if (  GoodTimeMsgSent == 1 )
                return( 1 );



        // Get current time to set year and day.

        time( &ltime );
        currtime = gmtime( &ltime );
        timestr  = asctime( currtime );
        

        // User has requested we skip this check;

        if ( WaitForGoodTime == WAITFORGOODTIME_NO ) {
                sprintf( MsgBuffer,
                         "Time not checked, sending data soon at %26s",
                         timestr );
                LogMessage( MsgBuffer, 0 );
                GoodTimeMsgSent = 1;
                return( 1 );
                }


        // Check time, send only two mesages.

        thisSec = (int)((PointTime) / ((SRINT64)(NSPERSEC)));
        
        if ( thisSec < MIN_GOOD_TIME ) {        // time NOT valid
                if ( ! BadTimeMsgSent ) {
                        sprintf( MsgBuffer,
                                 "Time is not valid yet,  no data sent at %26s",
                                 timestr );
                        LogMessage( MsgBuffer, 0 );
                        BadTimeMsgSent = 1;
                        }
                return( 0 );
                }
        else {                                  // time valid
                if ( ! GoodTimeMsgSent ) {
                        sprintf( MsgBuffer,
                                 "Time is valid now, sending data soon at %26s",
                                 timestr );
                        LogMessage( MsgBuffer, 0 );
                        GoodTimeMsgSent = 1;
                        }
                return( 1 );
                }
}

//------------------------------------------------------------------------------
// ROUTINE: ComputeInputRate
// PURPOSE: Compute the sampling rate based on the first two data point.
//          Sets the global sampling rate variables.  Exits on error.
//------------------------------------------------------------------------------
void ComputeInputRate( void ) {

        int          hundredMicro, usec;
        unsigned int intSec0, nanoSec0;
        double       Dperiod;        

        intSec0 = (unsigned int)( Pts[0].TimeStamp / (SRINT64)NSPERSEC );
        nanoSec0 = (unsigned int)(Pts[0].TimeStamp -
                                  ((SRINT64)intSec0 * (SRINT64)NSPERSEC) );
        
        if ( Pts[0].TimeStamp == Pts[1].TimeStamp )
                LogMessage( "First two points are identical (divide by zero!)\n", 1 );

        usec         = (int)(nanoSec0 / NSPERUS);
        hundredMicro = usec / 100;
        PrecisionSec = usec - ( hundredMicro * 100 );

        Dperiod      = ( (double)(Pts[1].TimeStamp - Pts[0].TimeStamp) / (double)NSPERSEC );
        Speriod      = (float)( Dperiod );
        Sps          = (float)( 1.0 / Dperiod );
}

//------------------------------------------------------------------------------
// ROUTINE: FinalOutput
// PURPOSE: Output any remaining points.
//------------------------------------------------------------------------------
void FinalOutput( void ) {

        SRINT64 startTime64;

        // Set true or dummy start time for last miniSeed files.

        if ( SampleCount > 0 ) { // actual start time
                startTime64  = Pts[0].TimeStamp;
                }
        else {                      // no points, use duumy time just past last valid end time
                startTime64 = Pts[MSEED_NSAMPLES-1].TimeStamp + 1;
                }
        
        // Output remaining points and any required padding to miniSeed files.

        if ( SampleCount >= 0 ) { 
                MiniSeedOpen( 0 ); // open for append
                MiniSeedOutputTitle( startTime64 );
                MiniSeedOutputData( SampleCount );
                MiniSeedOutputPad( (MSEED_NSAMPLES - SampleCount) );
                MiniSeedClose( );
                }
}

//------------------------------------------------------------------------------
// ROUTINE: CleanExit
// PURPOSE: Do all tasks needed for a clean exit.  These include closing the
//          output files, the input pipe and the report file.
//------------------------------------------------------------------------------
int CleanExit( int ExitCode ) {

        
        // Output remaining points and any required padding to miniSeed files.
        
        if ( SampleCount > 0 )
                FinalOutput( ); // closes output files


        // If the input pipe is not open, open it.  This is needed
        // so the pipeline will know to exit when this pipe is closed.

        if ( hInputPipe == SR_BADHANDLE ) {
                sprintf( InputPipeName, "%s%s", REQUIRED_PIPE_PREFIX, PROGRAMNAME );
                hInputPipe  = CustomOpenInputPipe( InputPipeName );
                }


        // Close the input pipe.

        if ( hInputPipe != SR_BADHANDLE )
                CustomCloseInputPipe( hInputPipe );

        
	// Close custom report.

        CustomReportClose( );


        // Goodbye.

        printf( "Bin2Seed: CleanExit with exit code %d\n", ExitCode );

        exit( ExitCode );
}

//------------------------------------------------------------------------------
// ROUTINE: LogMessage
// PURPOSE: Output message to both screen and report file.  If err is non-zero, 
//          then exit.
//------------------------------------------------------------------------------
void LogMessage( char *Msg, int Err ) {

        // Output message to screen and report file.

        printf( "Bin2Seed: %s", Msg );
        CustomReportPrint( Msg );


        // If error, then exit.

        if ( Err )
                CleanExit( Err );
}


// ****************************** MiniSeed Functions **********************
// ****************************** MiniSeed Functions **********************
// ****************************** MiniSeed Functions **********************

struct _Blockette100 { // Sample Rate Blockette
        SEEDUWORD Type;
        SEEDUWORD Next;
        SEEDFLOAT SampleRate;
        SEEDBYTE Flags;
        SEEDUBYTE Reserved[3];
        } Blockette100;

struct _Blockette1000 { // Data Only Blockette
        SEEDUWORD Type;
        SEEDUWORD Next;
        SEEDBYTE Encoding; // 3 = 32 bit integers
        SEEDBYTE WordOrder; // 0 = little endian
        SEEDBYTE DataLength;
        SEEDBYTE Reserved;
        } Blockette1000;

struct _Blockette1001 { // Data Extension Blockette
        SEEDUWORD Type;
        SEEDUWORD Next;
        SEEDBYTE TimingQuality; // 0-100%
        SEEDBYTE uSec; // 0 to 99, offset from std 100usec
        SEEDBYTE Reserved;
        SEEDBYTE FrameCount; //? compressed frames in 4K record, max 63
        } Blockette1001;

struct BTIME {
        SEEDUWORD Year;
        SEEDUWORD JulianDay;
        SEEDUBYTE Hour;
        SEEDUBYTE Minute;
        SEEDUBYTE Second;
        SEEDUBYTE Unused;
        SEEDUWORD HundredMicrosec;
        };

struct _DataHdr {
        char SeqNum[6];
        char Quality;              // R = raw
        char Reserved;             // space
        char Station[5];
        char Location[2];
        char Channel[3];
        char Network[2];
        struct BTIME StartTime;
        SEEDUWORD NumSamples;
        SEEDWORD SampleRateFactor;     // > 0 sps, < 0 sec/samp
        SEEDWORD SampleRateMultiplier; // > 0 multiply, < 0 divide
        SEEDUBYTE ActivityFlags;       // bits 0-6 = cal, time corr applied,trigger,endtrigger,+leapsec,-leapsec,event in progress
        SEEDUBYTE IoClockFlags;        // bits 0-5=parity err, long rec, short rec, start series, end series, clock locked
        SEEDUBYTE QualityFlags;        // bits 0-7=ampsaturated, dig clip, spikes, glitched, missing/padded data, synch err, dig filter charging, time tag questionable
        SEEDUBYTE NumBlockettes;
        SEEDLONG TimeCorrection;       // in .0001 sec (100us)
        SEEDUWORD DataBegin;
        SEEDUWORD FirstBlockette;
        } DataHdr;


FILE *fpChannelOutData[MAXCHANNELS];   // global array of file pointers, one for each channel


#define MSEED_ENDIAN          1 // 0 = little endian (VAX/PC), 1 = big endian (required by SeedLink)
#define MSEED_LENGTH_EXP      9
#define MSEED_PACKET_SIZE   512 // 2**MSEED_LENGTH_EXP



#define MSEED_ACTIVITY_NONE                 0x00
#define MSEED_ACTIVITY_HAS_CALIBRATION      0x01
#define MSEED_ACTIVITY_TIME_CORR_APPLIED    0x02
#define MSEED_ACTIVITY_EVENT_BEGIN          0x04
#define MSEED_ACTIVITY_EVENT_END            0x08
#define MSEED_ACTIVITY_LEAPSEC_POSITIVE     0x10
#define MSEED_ACTIVITY_LEAPSEC_NEGATIVE     0x20
#define MSEED_ACTIVITY_EVENT_IN_PROGRESS    0x40

#define MSEED_IOCLOCK_NONE                  0x00
#define MSEED_IOCLOCK_PARITY_ERROR          0x01
#define MSEED_IOCLOCK_LONG_RECORD           0x02
#define MSEED_IOCLOCK_SHORT_RECORD          0x04
#define MSEED_IOCLOCK_TIME_SERIES_START     0x08
#define MSEED_IOCLOCK_TIME_SERIES_END       0x10
#define MSEED_IOCLOCK_CLOCK_LOCKED          0x20

#define MSEED_QUALITY_NONE                  0x00
#define MSEED_QUALITY_AMP_SATURATED         0x01
#define MSEED_QUALITY_DIGITIZER_CLIPPED     0x02
#define MSEED_QUALITY_SPIKES_DETECTED       0x04
#define MSEED_QUALITY_GLITCHES_DETECTED     0x08
#define MSEED_QUALITY_MISSING_DATA          0x10
#define MSEED_QUALITY_TELEMETRY_SYNCH_ERR   0x20
#define MSEED_QUALITY_FILTER_CHARGING       0x40
#define MSEED_QUALITY_TIME_TAG_QUESTIONABLE 0x80

//struct BTIME {
//    SEEDUWORD Year;             /* year, eg. 2003                   */
//    SEEDUWORD JulianDay;        /* day of year (1-366)              */
//    SEEDUBYTE Hour;             /* hour (0-23)                      */
//    SEEDUBYTE Minute;           /* minute (0-59)                    */
//    SEEDUBYTE Second;           /* second (0-59), 60 if leap second */
//    SEEDUBYTE Unused;    
//    SEEDUWORD HundredMicrosec;  /* microsecond x100 (0-9999)        */

//struct tm {
//    int tm_sec;                 /* seconds (0-59)                   */
//    int tm_min;                 /* minutes (0-59)                   */
//    int tm_hour;                /* hours (0-59)                     */
//    int tm_mday;                /* day of the month (1-31)          */
//    int tm_mon;                 /* month (0-11)                     */
//    int tm_year;                /* year (since 1900)                */
//    int tm_wday;                /* day of the week (0-6; 0=sunday)  */
//    int tm_yday;                /* day in the year (0-365)          */
//    int tm_isdst;               /* daylight saving time             */

//------------------------------------------------------------------------------
// ROUTINE: MakeBtime
// PURPOSE: Combine number of seconds since 1970 plus number of microseconds
//          into the SEED BTIME structure.
//------------------------------------------------------------------------------
int MakeBtime( int IntSec1970, int NanoSec1970, struct BTIME *SampleBtime ) {

        struct tm *tmtime;
        time_t     tt;
        short      year, jday, cus;
        int        usec;

        if ( !SampleBtime )
                return( 0);

        tt   = (time_t)IntSec1970;
        usec = (int)( NanoSec1970 / NSPERUS );
         
        tmtime = gmtime( &tt ); // UTC time

        year                         = tmtime->tm_year+1900;
        jday                         = tmtime->tm_yday+1;
        cus                          = usec / 100; // rounds down

        // Compute new precision offset for each new title block.
        
        PrecisionSec              = usec - ( cus * 100 );
        
        SampleBtime->Year            = SwapInt2( year );
        SampleBtime->JulianDay       = SwapInt2( jday );
        SampleBtime->Hour            = tmtime->tm_hour;
        SampleBtime->Minute          = tmtime->tm_min;
        SampleBtime->Second          = tmtime->tm_sec;
        SampleBtime->Unused          = 0;
        SampleBtime->HundredMicrosec = SwapInt2( cus );

        return( 1 );
}

//------------------------------------------------------------------------------
// ROUTINE: SwapInt2
// PURPOSE: Swap the bytes of a 2 byte short integer.  Return the swapped short.
//------------------------------------------------------------------------------
short SwapInt2( short DataIn ) {
        
        short          dataOut;
        unsigned char *byteIn, *byteOut;


        byteIn  = (char*)(&DataIn);
        byteOut = (char*)(&dataOut);

        byteOut[0] = byteIn[1];
        byteOut[1] = byteIn[0];

        return( dataOut );
}

//------------------------------------------------------------------------------
// ROUTINE: SwapInt4
// PURPOSE: Swap the bytes of a 4 byte integer.  Return the swapped integer.
//------------------------------------------------------------------------------
int SwapInt4( int DataIn ) {
        
        int            i, dataOut;
        unsigned char *byteIn, *byteOut;


        byteIn  = (char*)(&DataIn);
        byteOut = (char*)(&dataOut);

        for ( i = 0 ; i < 4 ; i++ )
                byteOut[i] = byteIn[3-i];

        return( dataOut );
}

//------------------------------------------------------------------------------
// ROUTINE: SwapFloat
// PURPOSE: Swap the bytes of a 4 byte float.  Return the swapped float.
//------------------------------------------------------------------------------
float SwapFloat( float DataIn ) {
        
        int            i;
        unsigned char *byteIn, *byteOut;
        float          dataOut;


        byteIn  = (char*)(&DataIn);
        byteOut = (char*)(&dataOut);

        for ( i = 0 ; i < 4 ; i++ )
                byteOut[i] = byteIn[3-i];

        return( dataOut );
}

//------------------------------------------------------------------------------
// ROUTINE: FillChars
// PURPOSE: Fill the specified number of characters from the provided string
//          into the provided location.  Returns 1 for success, 0 on error.
//------------------------------------------------------------------------------
int FillChars( int NumChar, char *Src, char *Dest ) {

        int i;

        if ( !Src || !Dest )
                return( 0 );
        
        for ( i = 0 ; i < NumChar ; i++ )
                Dest[i] = Src[i];

        return( 1 );
}


//------------------------------------------------------------------------------
// ROUTINE: MiniSeedOpen
// PURPOSE: Open one miniSEED file per channel.
//------------------------------------------------------------------------------
void MiniSeedOpen( int FirstOpen ) {

        // Open one mini-SEED file per channel.

        static int lastjday = -1;        
        int   ichan, jday, year, sysret;
        char *openMode, channelFileName[MAXFILENAME], Command[MAXFILENAME];
        time_t ltime;
        struct tm *currtime;


        // Get current time to set year and day.

        time( &ltime );
        currtime = gmtime( &ltime );
        
        year = currtime->tm_year + 1900;
        jday = currtime->tm_yday + 1;



        // On first open, prepare names and create the directories
        // for each channel's data.

        if ( FirstOpen ) {

                // For each channel ...

                for ( ichan = 0 ; ichan < Nchannels ; ichan++ ) {

                        // Prepare directory name.

                        if ( NameStyle == NAMESTYLE_SDS ) {
                                sprintf( ChNameDir[ichan],
                                         ".%c%4u%c%s%c%s%c%s.%c",
                                                           
                                         SR_DIRSLASH, year,
                                         SR_DIRSLASH, ChNameNet[ichan],
                                         SR_DIRSLASH, ChNameSta[ichan],
                                         SR_DIRSLASH, ChNameCmp[ichan], CHAN_TYPE
                                       );
                                }

                        else { // if ( NameStyle == NAMESTYLE_BUD )
                                sprintf( ChNameDir[ichan],
                                         ".%c%s%c%s",
                                         SR_DIRSLASH, ChNameNet[ichan],
                                         SR_DIRSLASH, ChNameSta[ichan]
                                       );
                                }


                        // Make the directory.

                        sprintf( Command, "mkdir %s %s", SR_MKDIR_OPTS, ChNameDir[ichan] );
                        
                        sysret = system( Command );
                        if ( sysret == -1 ) {
                                sprintf( MsgBuffer,
                                         "System call error %d, errno = %d\nFailed to run command '%s'\n",
                                         sysret, errno, Command );
                                LogMessage( MsgBuffer, 1 );  // < fatal ...
                                }

                        } // end for ichan < Nchannels
                
                } // end if FirstOpen


        // Prepare file names when the Julian day has changed.

        if ( lastjday != jday ) {

                // For each channel ...

                for ( ichan = 0 ; ichan < Nchannels ; ichan++ ) {

                        // Prepare the file name.

                        if ( NameStyle == NAMESTYLE_SDS ) {
                                sprintf( ChNameFil[ichan],
                                         "%s.%s.%s.%s.%c.%04u.%03u",
                                         ChNameNet[ichan],
                                         ChNameSta[ichan],
                                         ChNameLoc[ichan],
                                         ChNameCmp[ichan],
                                         CHAN_TYPE,
                                         year,
                                         jday
                                       );
                                }

                        else { // if ( NameStyle == NAMESTYLE_BUD ) 
                                sprintf( ChNameFil[ichan],
                                         "%s.%s.%s.%s.%04u.%03u",
                                         ChNameSta[ichan],
                                         ChNameNet[ichan],
                                         ChNameLoc[ichan],
                                         ChNameCmp[ichan],
                                         year,
                                         jday
                                       );
                                }

                        } // end if for ichan < Nchannels
                
                lastjday = jday;
                openMode = "wb";        // open for write

                } // end if Julian day has changed

        else { // julian day has not changed
        
                openMode = "ab";        // open for append
                }

                

        // Open the output files for mini-seed data.

        openMode = "ab";        // always open for append
        
        for ( ichan = 0 ; ichan < Nchannels ; ichan++ ) {

                sprintf( channelFileName, "%s%c%s",
                         ChNameDir[ichan], SR_DIRSLASH, ChNameFil[ichan] );

                fpChannelOutData[ichan] = fopen( channelFileName, openMode );

                if ( !fpChannelOutData[ichan] ) {

                        sprintf( MsgBuffer, "failed to open mini-seed channel output file %s\n", channelFileName );
                        LogMessage( MsgBuffer, 1 );  // < fatal ...
                        }

                } // end for ichan
}

//------------------------------------------------------------------------------
// ROUTINE: MiniSeedClose
// PURPOSE: Close one miniSEED file per channel.
//------------------------------------------------------------------------------
void MiniSeedClose( void ) {
        
        int ichan;

        // Close one mini-SEED file per channel.

        for ( ichan = 0 ; ichan < Nchannels ; ichan++ ) {
                fclose( fpChannelOutData[ichan] );
                }
}

//------------------------------------------------------------------------------
// ROUTINE: MiniSeedOutputTitle
// PURPOSE: Write out one miniSEED format title block per channel.
//------------------------------------------------------------------------------
void MiniSeedOutputTitle( SRINT64 StartTime64 ) {

        int          ichan, Sps10Diff;
        SEEDUBYTE    activity, ioClock, quality;
        struct BTIME fileBtime;
        short        numSamples, sampRateFact, sampRateMult;
        short        firstBlock, secondBlock, thirdBlock, dataBegin;
        unsigned int intSec1970, nanoSec1970;
        char         srcString[MAXSTR];


        // Compute start time as a Btime.

        intSec1970 = (unsigned int)( Pts[0].TimeStamp / (SRINT64)NSPERSEC );
        nanoSec1970 = (unsigned int)(Pts[0].TimeStamp -
                                  ((SRINT64)intSec1970 * (SRINT64)NSPERSEC) );

        MakeBtime( intSec1970, nanoSec1970, &fileBtime );


        // Assign various fields.

        activity = MSEED_ACTIVITY_NONE;
        ioClock  = MSEED_IOCLOCK_NONE;
        quality  = MSEED_QUALITY_NONE;

        if ( CurrentPoint == 0 )
                ioClock   |= MSEED_IOCLOCK_TIME_SERIES_START;
        
        if ( SampleCount == MSEED_NSAMPLES ) {
                numSamples = MSEED_NSAMPLES;
                }
        else {
                numSamples = (short)SampleCount;
                ioClock   |= MSEED_IOCLOCK_TIME_SERIES_END;
                }


        // Compute blockette locations.

        firstBlock   = (short)(sizeof(DataHdr));
        secondBlock  = (short)(firstBlock + sizeof(Blockette100));
        thirdBlock   = (short)(secondBlock + sizeof(Blockette1001));
        dataBegin    = (short)(thirdBlock + sizeof(Blockette1000));


        // Compute sample rate factor and multiplier.  Allow a single
        // correction by 10 as long as the rate wouldn't overflow a short.

        sampRateFact = (short)(Sps);
        sampRateMult = (short)(1);
        Sps10Diff    = ((int)(10.0*Sps) - (10*sampRateFact));

        if ( (Sps10Diff > 0) & (Sps < 3000) ) {
                sampRateFact = (short)(Sps*10.0);
                sampRateMult = (short)(-10);
                }


        // Fill in the Fixed Data Header.

        sprintf( srcString, "%06d", FileNumber );
        FillChars( 6, srcString, DataHdr.SeqNum );             // char*6
        DataHdr.Quality              = 'R';                    // char D = undefined, R = raw
        DataHdr.Reserved             = ' ';                    // char space
        FillChars( 5, "CH0X",  DataHdr.Station  );             // char*5
        FillChars( 2, "CX",    DataHdr.Location );             // char*2
        FillChars( 3, "SHZ",   DataHdr.Channel  );             // char*3
        FillChars( 2, "XX",    DataHdr.Network  );             // char*2
        DataHdr.StartTime            = fileBtime;              // struct BTIME 
        DataHdr.NumSamples           = SwapInt2( numSamples ); // SEEDUWORD 
        DataHdr.SampleRateFactor     = SwapInt2(sampRateFact); // SEEDWORD > 0 sps, < 0 sec/samp
        DataHdr.SampleRateMultiplier = SwapInt2(sampRateMult); // SEEDWORD > 0 multiply, < 0 divide
        DataHdr.ActivityFlags        = activity;               // SEEDUBYTE
        DataHdr.IoClockFlags         = ioClock;                // SEEDUBYTE
        DataHdr.QualityFlags         = quality;                // SEEDUBYTE
        DataHdr.NumBlockettes        = 3;                      // SEEDUBYTE 2 Blockettes follows (#100+#1001+Data)
        DataHdr.TimeCorrection       = 0;                      // SEEDLONG in .0001 sec (100us)
        DataHdr.DataBegin            = SwapInt2( dataBegin);   // SEEDUWORD 
        DataHdr.FirstBlockette       = SwapInt2( firstBlock ); // SEEDUWORD


        // Fill in sample rate blockette as variable header.         
        
        Blockette100.Type          = SwapInt2( 100 );         // SEEDUWORD 
        Blockette100.Next          = SwapInt2( secondBlock ); // SEEDUWORD 
        Blockette100.SampleRate    = SwapFloat( Sps );        // SEEDFLOAT 
        Blockette100.Flags         = 0;                       // SEEDBYTE 
        Blockette100.Reserved[0]   = 0;                       // SEEDUBYTE 
        Blockette100.Reserved[1]   = 0;                       // SEEDUBYTE 
        Blockette100.Reserved[2]   = 0;                       // SEEDUBYTE 


        // Fill in data blockette 1001 header.
        
        Blockette1001.Type          = SwapInt2( 1001 );       // SEEDUWORD
        Blockette1001.Next          = SwapInt2( thirdBlock ); // SEEDUWORD 
        Blockette1001.TimingQuality = 100;                    // SEEDBYTE 0 to 100%
        Blockette1001.uSec          = PrecisionSec;           // SEEDBYTE 0 to 99, offset from std 100usec
        Blockette1001.Reserved      = 0;                      // SEEDBYTE
        Blockette1001.FrameCount    = 0;                      // SEEDBYTE compressed frames in 4K record, max 63


        
        // Fill in data blockette 1000 header.
        
        Blockette1000.Type          = SwapInt2( 1000 );       // SEEDUWORD
        Blockette1000.Next          = 0;                      // SEEDUWORD no more data blockettes follow
        Blockette1000.Encoding      = 3;                      // SEEDBYTE 3 = 32 bit integers
        Blockette1000.WordOrder     = MSEED_ENDIAN;           // SEEDBYTE 0 = little endian (VAX/PC), 1 = big endian
        Blockette1000.DataLength    = MSEED_LENGTH_EXP;       // SEEDUBYTE
        Blockette1000.Reserved      = 0;                      // SEEDBYTE


        
        // Update headers and output to all channel files.

        for ( ichan = 0 ; ichan < Nchannels ; ichan++ ) {

                // Update with channel info from config file.
                
                FillChars( 5, ChNameSta[ichan], DataHdr.Station  );  // char*5
                FillChars( 2, ChNameLoc[ichan], DataHdr.Location );  // char*2
                FillChars( 3, ChNameCmp[ichan], DataHdr.Channel  );  // char*3
                FillChars( 2, ChNameNet[ichan], DataHdr.Network  );  // char*2

                // Output to channel files.
                
                fwrite( &DataHdr,       1, sizeof( DataHdr ),       fpChannelOutData[ichan] );
                fwrite( &Blockette100,  1, sizeof( Blockette100 ),  fpChannelOutData[ichan] );
                fwrite( &Blockette1001, 1, sizeof( Blockette1001 ), fpChannelOutData[ichan] );
                fwrite( &Blockette1000, 1, sizeof( Blockette1000 ), fpChannelOutData[ichan] );
               }


        // Prepare for next iteration.

        FileNumber++;
        if ( FileNumber == 1000000 ) // only space for max of 6 digits
                FileNumber = 1;
}

//------------------------------------------------------------------------------
// ROUTINE: MiniSeedOutputData
// PURPOSE: Write out one miniSEED format data block per channel.
//------------------------------------------------------------------------------
void MiniSeedOutputData( int NumPoints ) {

        int ichan, ipt, nbytes, dataValue;


        // Quick exit if nothing to do.
        
        if ( NumPoints == 0 )
                return;

        // Output all data points for each channel.

        for ( ipt = 0 ; ipt < NumPoints ; ipt++ ) {

                for ( ichan = 0 ; ichan < Nchannels ; ichan++ ) {

                        // Swap bytes if required.

                        if ( MSEED_ENDIAN == 0 )  // want little endian (PC), already correct
                                dataValue = Pts[ipt].Analog[ichan];    

                        else                      // want big endian (SPARC), swap required
                                dataValue = SwapInt4( Pts[ipt].Analog[ichan] );


                        nbytes = fwrite( &dataValue, 1, sizeof(int), fpChannelOutData[ichan] );

                        if ( nbytes != sizeof(int) ) {

                                sprintf( MsgBuffer,
                                         "failed to write data point for sample %s, channel %d",
                                         CustomPrint64( Pts[ipt].Sample ), ichan );
                                LogMessage( MsgBuffer, 1 );  // < fatal ...
                                }

                        } // end for ichan

                CurrentPoint++;

                } // end for ipt
}

//------------------------------------------------------------------------------
// ROUTINE: MiniSeed Output Pad
// PURPOSE: Write out one miniSEED format padding per channel.
//------------------------------------------------------------------------------
void MiniSeedOutputPad( int NumPoints ) {

        int   i, ichan, nbytes, npadbytes, padValues[MSEED_NSAMPLES];
        char *pv;


        // Quick exit if nothing to do.
        
        if ( NumPoints == 0 )
                return;

        // Initialize pad size and values.

        npadbytes = NumPoints * sizeof( int );
        pv        = (char*)padValues;
        for ( i = 0 ; i < npadbytes ; i++ ) 
                *pv++ = '\0';

        
        // We've run out of real points so pad out to required number of
        // samples for all channels.

       for ( ichan = 0 ; ichan < Nchannels ; ichan++ ) {

                nbytes = fwrite( padValues, 1, npadbytes, fpChannelOutData[ichan] );

                if ( nbytes != npadbytes ) {

                        sprintf( MsgBuffer,
                                 "failed to write %d pad values for channel %d",
                                 NumPoints, ichan );
                        LogMessage( MsgBuffer, 1 );  // < fatal ...
                        }

               } // end for ichan
}

